#!/usr/bin/perl -w

##  Copyright 2003 Rien Croonenborghs
##
##    This file is part of file2divx3pass.
##    file2divx4pass is free software; you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation; either version 2 of the License, or
##    (at your option) any later version.
##    file2divx3pass is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##    You should have received a copy of the GNU General Public License
##    along with file2divx3pass; if not, write to the Free Software
##    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use strict;

package File2DiVX3Pass;			## belongs to this package
use File2DiVX3Pass::interface;		## makes use of interface -> has constructor
use File2DiVX3Pass::signals;		## makes use of signals -> for connecting the widgets

my $true  = 1;
my $false = 0;

my %specs = ();
my %settings = ( 	'status' => "stopped",
			'batchstatus' => "stopped",
			'mencoderpid' => -1,
			'divx4extra' => "q=5:min_quant=2:max_quant=8",
			'lavcextra' => "vhq:vqmin=2:vqmax=2:vme=1:keyint=25"		);
my $error = undef;

## USEFULL ---------------------------------------------------------------------

## debug to stdout
sub Debug
{
	my ($msg) = @_;
	print STDOUT "$msg\n";
}

## print message in actions label
sub MyPrint
{
	my ($widgets,$msg) = @_;
	$widgets->{'lblActions'}->set_text( $msg );
}

## ERRORS set and get
sub SetError
{
	my ($msg) = @_;
	$error = $msg;
}
sub GetError
{
	return $error;
}

## CLEANSTRING -> acts like chop
#sub CleanString
#{
#	my ($txt) = @_;
#	$_ = $txt; s/\n//g; $txt = $_; return $txt;
#}

## check if settings dir exists
sub SettingsDirExists
{
	## get homedir
	my $home = $ENV{HOME};
	## check if .file2divx3pass dir exists
	if( -e $home."/.file2divx3pass" && -d $home."/.file2divx3pass" )
	{ return $true; }
	else { return $false; }

	#opendir(DIR,$home);
	#my @f = readdir(DIR);
	#closedir(DIR);
	#foreach my $entry( @f )
	#{
	#	if( $entry eq ".file2divx3pass" )
	#	{
	#		return $true;
	#	}
	#}
	#return $false;
}

## check if settings file exists
sub SettingsFileExists
{
	## get homedir
	my $home = $ENV{HOME};
	## check if .file2divx3pass/settigngs file exists
	if( -e $home."/.file2divx3pass/settings" && -f $home."/.file2divx3pass/settings" )
	{ return $true; }
	else { return $false; }

	#if( SettingsDirExists() )
	#{
	#	## get homedir
	#	my $home = $ENV{HOME};
	#	## check if settingsfile exists in home/settingsdir
	#	opendir( DIR, $home."/.file2divx3pass" );
	#	my @f = readdir(DIR);
	#	closedir(DIR);
	#	foreach my $entry( @f )
	#	{
	#		if( $entry eq "settings" )
	#		{
	#			return $true;
	#		}
	#	}
	#}
	#return $false;
}

## FILE TOOLS ------------------------------------------------------------------

## FILESPECS
############
sub ReadFileSpecs
{
	my ($file) = @_;

	## clear the previous specs
	%specs = ();

	## prepare cmd to be run
	my $cmd = "mplayer -identify -vc null -ac null -frames 0 \"$file\"";
	## open
	my $res = open( TMP, "$cmd 2>&1 |" );
	if( !defined($res) )
	{
		SetError( "Could not identify $file" );
		return $false;
	}
	## scan
	while( <TMP> )
	{
		## frames/s
		if( $_ =~ /ID_VIDEO_FPS=(\d+\.\d+)/ )
		{
			$specs{'fps'} = $1;
		}
		## some locales use the comma as decimal symbol
		elsif( $_ =~ /ID_VIDEO_FPS=(\d+\,\d+)/ )
		{
			$specs{'fps'} = $1;
			$specs{'fps'} =~ s/\,/\./g;
		}
		## length
		## realmedia will return 0 no mather what -> so indicate with 59:59:59 time
		elsif( $_ =~ /ID_LENGTH=(\d+)/ )
		{
			$specs{'seconds'} = $1;
			if( $specs{'seconds'} eq "0" ) { $specs{'seconds'} = "215999"; }

		}
		elsif( $_ =~ /ID_VIDEO_BITRATE=(\d+)/ )
		{
			$specs{'vbitrate'} = $1;
		}
	}
	## close
	close( TMP );

	## check the fps and seconds
	if( !defined($specs{'fps'}) )
	{
		SetError( "No frames/s found; make sure this is a valid media-file." );
		return $false;
	}
	elsif( !defined($specs{'seconds'}) )
	{
		SetError( "No length found; make sure this is a valid media-file." );
		return $false;
	}
	else { return $true; }
}
## return the spec fps
sub GetSpecFPS
{
	return $specs{'fps'};
}
## return the spec length in seconds
sub GetSpecSeconds
{
	return $specs{'seconds'};
}
sub GetSpecVBitrate
{
	return $specs{'vbitrate'} || 1;
}


## CALCULATE THINGS :) ---------------------------------------------------------

## CALCULATE VIDEOBITRATE
#########################
sub CalcVBrate
{
	my ($cdsize,$minutes,$audiobrate) = @_;
	my $videobrate = (((($cdsize*1024*8)/1000)*1024)-($audiobrate*$minutes*60)) / ($minutes*60) ;
	$videobrate = substr( $videobrate, 0, index( $videobrate, "." ) );
	# taken from divxcalc 0.5 (http://axljab.homelinux.org:8080/?m=sw&p=divxcalc)
}


## CALCULATE BPP
################
sub CalcBPP
{
	my ($videobrate,$width,$height,$fps) = @_;
	$fps  =~ s/\,/\./g; ## locales decimal
	my $bpp = ($videobrate*1000) / ($width*$height*$fps);
	return $bpp;
}

## CHECK ENTRIES ---------------------------------------------------------------

## CHECK ENTRY
##############
## only [0-9]
sub CheckNumeralEntry
{
	my ($entry) = @_;
	my $txt = $entry->get_text;
	## no empty
	if( $txt eq "" )
	{ SetError("No value"); return $false; }
	## only 0-9
	elsif( $txt =~ /[A-Z,a-z]/ || $txt =~ /,;:=?\/+~%'`\[\]\$\*&\"\'(!)-_|@#{}<>\\/ )
	{ SetError("Only numerals"); return $false; }
	else{ return $true; }
}
## only non [0-9] characters
sub CheckNonNumeralEntry
{
	my ($entry) = @_;
	my $txt = $entry->get_text;
	## no empty
	if( $txt eq "" )
	{ SetError("No value"); return $false; }
	else{ return $true; }
}
## hh:mm:ss formatted hour
sub CheckHourEntry
{
	my ($entry) = @_;
	my $txt = $entry->get_text;
	## no empty
	if( $txt eq "" )
	{ SetError("No value"); return $false; }
	my ($h,$m,$s) = split( /:/, $txt );

	if( !( $h =~ /([0-9]{2})/ ) ) { SetError( "Wrong hour value" ); return $false; }
	if( !( $m =~ /([0-9]{2})/ ) ) { SetError( "Wrong minute value" ); return $false; }
	if( !( $s =~ /([0-9]{2})/ ) ) { SetError( "Wrong second value" ); return $false; }

	return $true;
}

## ENCODING --------------------------------------------------------------------

## ENCODE VIDEO : prepare the command(s) to runs
sub EncodeVideo
{
	my ( $infile,		## input file
		$subfile,		## subtitle file = -1=none, path
		$subid,		## subtitle id : -1=none, >0
		$audioid,		## audio id : -1=none, >0
		$outfile,		## output file
		$audiocodec,	## audio codec : mp3, copy, pcm
		$audiobrate,	## audio mp3 bitrate
		$audiorate,	## audio rate : constant, averag
		$width,		## scale width
		$height,		## scale height
		$videobrate,	## video bitrate
		$videocodec,	## video codec : lavc, divx4, copy
		$pass,		## number of passes to perform : 1-pass, 2-pass, 3-pass
		$encopts,		## extra encoding options
		$ofps,		## forced output frames/s : -1=none, >0
		$widgets )   = @_;

	## make audio options
	my $cmdaudio = "";
	if( $audiocodec eq "mp3" )
	{
		$cmdaudio = " -oac mp3lame -lameopts ";
		## rate
		if( $audiorate eq "constant" ) { $cmdaudio .= "cbr:"; }
		elsif( $audiorate eq "average" ) { $cmdaudio .= "abr:"; }
		## bitrate
		$cmdaudio .= "br=$audiobrate ";
	}
	elsif( $audiocodec eq "copy" )
	{
		$cmdaudio = " -oac copy ";
	}
	elsif( $audiocodec eq "pcm" )
	{
		$cmdaudio = " -oac pcm ";
	}

	## make vop options
	my $cmdvop = " -vop pp=de,scale=".$width.":".$height." ";

	## make output FPS options
	my $cmdofps = "";
	if( $ofps != -1 )
	{
		$ofps  =~ s/\,/\./g; ## locales decimal
		$cmdofps = " -ofps $ofps ";
	}

	## make codec options
	my $cmdcodec = "";
	if( $videocodec eq "divx4" )
	{
		$cmdcodec = " -ovc divx4 -divx4opts $encopts:br=$videobrate:pass";
	}
	elsif( $videocodec eq "lavc" )
	{
		$cmdcodec = " -ovc lavc -lavcopts $encopts:vbitrate=$videobrate:vpass";
	}
	elsif( $videocodec eq "copy" )
	{
		$cmdcodec = " -ovc copy ";
	}


	## make subtitle + audio id options
	my $cmdsubaudioids = "";
	if( $subid ne "-1" )
	{
		$cmdsubaudioids .= " -sid $subid ";
	}
	if( $audioid ne "-1" )
	{
		$cmdsubaudioids .= " -aid $audioid ";
	}

	## make subtitle file options
	my $cmdsubfile = "";
	if( $subfile ne "-1" )
	{
		$cmdsubfile = " -sub \"$subfile\" ";
	}


	## make all the commands
	## is an array -> 0-2 for 1, 2 or 3 pass encoding
	my @cmds = ();
	if( $pass eq "1-pass" && $videocodec ne "copy" )
	{
		$cmds[0] = "mencoder $cmdsubaudioids $cmdofps $cmdvop $cmdaudio $cmdcodec=1 $cmdsubfile -o \"$outfile\" \"$infile\"";
	}
	elsif( $pass eq "2-pass" && $videocodec ne "copy" )
	{
		$cmds[0] = "mencoder $cmdsubaudioids $cmdofps $cmdvop $cmdaudio $cmdcodec=1 $cmdsubfile -o /dev/null \"$infile\"";
		$cmds[1] = "mencoder $cmdsubaudioids $cmdofps $cmdvop $cmdaudio $cmdcodec=2 $cmdsubfile -o \"$outfile\" \"$infile\"";
	}
	elsif( $pass eq "3-pass" && $videocodec ne "copy" )
	{
		$cmds[0] = "mencoder $cmdsubaudioids $cmdofps -ovc frameno $cmdaudio $cmdsubfile -o frameno.avi \"$infile\"";
		$cmds[1] = "mencoder $cmdsubaudioids $cmdofps $cmdvop -oac copy $cmdcodec=1 $cmdsubfile -o /dev/null \"$infile\"";
		$cmds[2] = "mencoder $cmdsubaudioids $cmdofps $cmdvop -oac copy $cmdcodec=2 $cmdsubfile -o \"$outfile\" \"$infile\"";
	}
	elsif( $videocodec eq "copy" )
	{
		@cmds = ();
		$cmds[0] = "mencoder $cmdsubaudioids $cmdaudio $cmdcodec -o \"$outfile\" \"$infile\"";
	}

	Clean();
	my $encodingresult = $true;
	for( my $i=0; $i<@cmds; ++$i )
	{
		if( GetStatus() eq "started" )
		{
			if( !EncodePass( $cmds[$i], $i+1, $widgets ) )
			{
				SetError( GetError()." while encoding pass ".($i+1) );
				$encodingresult = $false;
				return $false;
			}
		}
		else
		{
			SetError( "Encoding stopped" );
			return $false;
		}
	}
	## only clean up after encoding is encoding was successful
	Clean() if( $encodingresult );

	return $encodingresult;
}

## encode the pass with command attached
sub EncodePass
{
	my ($cmd,$pass,$widgets) = @_;
	print "$cmd\n";


	## set label and reset progress bar
	$widgets->{'lblActions'}->set_text( "Encoding pass $pass" );
	$widgets->{pbarBar}->set_value( 0 );
	while (Gtk->events_pending()) {Gtk->main_iteration()};

	## get spec fps and seconds
	## if the time in entryLength != spec length, the time of entryLength is used
	my $specfps = GetSpecFPS();
	my $specsec = GetSpecSeconds();
	$specsec = HtoS( $widgets->{'entryLength'}->get_text ) if( HtoS( $widgets->{'entryLength'}->get_text ) ne $specsec );

	## execute command
	my $res = open( TMP, "$cmd 2>&1 |" );
	if( !defined($res) )
	{
		SetError( "Could not encode pass $pass" );
		return $false;
	}
	## remember the mencoder pid -> for stopping the process
	SetMencoderPID( $res );

	$/="\r"; ## from http://acidrip.thirtythreeandathird.net/
	## loop and parse the output of mencoder
	while( <TMP> )
	{
		#Pos:  49.0s   1179f ( 1%) 232fps Trem:   8min  69mb  A-V:0.084 [0:115]
		if( $_ =~ /Pos:\s{0,2}(.+)s\s+(\d+)f\s\(\s?(\d+)%\)\s*(\d+)fps\sTrem:\s+(\d+)min\s+(\d+)mb\s+A-V:(\d+\.\d+)\s\[(\d+):(\d+)\]/ )
		{
			my $numframes = $2;
			my $pct = $3;
			my $fps = $4;
			$fps  =~ s/\,/\./g; ## locales decimal
			#my $eta = $5;
			#my $efs = $6; # estimated file size
			my $todotime = StoH( $specsec - ($numframes/$specfps) );
			$todotime  =~ s/\,/\./g; ## locales decimal
			my $donetime = StoH( $numframes / $specfps );
			$donetime  =~ s/\,/\./g; ## locales decimal 

			my $str = sprintf( "Encoding pass %d - fps: %4d - done: %s - todo: %s", $pass, $fps, $donetime, $todotime );

			## update the label or the progress bar if necessary
			if ( $widgets->{'pbarBar'}->get_value ne $pct )
			{
				$widgets->{'pbarBar'}->set_value( $pct ) ;
			}
			if( $widgets->{'lblActions'}->get ne $str )
			{
				$widgets->{'lblActions'}->set_text( $str )
			}
			while (Gtk->events_pending()) { Gtk->main_iteration()};
		}
		while (Gtk->events_pending()) { Gtk->main_iteration()};
	}
	$/="\n"; ## from http://acidrip.thirtythreeandathird.net/
	## close
	close( TMP );

	## set progress bar to 100% only if no encoding is stopped
	## if stopped the progress bar stops indicating where it should
	$widgets->{pbarBar}->set_value( 100 ) if( GetStatus() eq "started" );

	## reset the mencoder pid
	SetMencoderPID( undef );
	return $true;
}

## clean up temporary files
sub Clean
{
	unlink "frameno.avi", "divx2pass.log", "analyse.log", "c:\\trace_b.txt";
}


## HOUR<->SECONDS --------------------------------------------------------------

## hh:mm:ss to seconds
sub HtoS
{
	my ($h) = @_;

	my ($hh,$mm,$ss) = split( /:/, $h );
	return ($hh*3600) + ($mm*60) + $ss;
}

## seconds to hh:mm:ss
sub StoH
{
	my ($s) = @_;

	if( $s < 0 ) { return "neg. time"; }

	my $hh = 0;
	my $mm = 0;

	if( $s / 3600 > 0 )
	{
		$hh = $s / 3600;
		$hh = substr( $hh, 0, index($hh,".") ) if( $hh =~ /\./ );
		$s = $s % 3600;
	}
	if( $s / 60 > 0 )
	{
		$mm = $s / 60;
		$mm = substr( $mm, 0, index($mm,".") ) if( $mm =~ /\./ );
		$s = $s % 60;
	}
	for( my $i=length($hh); $i<2; ++$i ) { $hh = "0".$hh; }
	for( my $i=length($mm); $i<2; ++$i ) { $mm = "0".$mm; }
	for( my $i=length($s); $i<2; ++$i ) { $s = "0".$s; }
	return "$hh:$mm:$s";
}

## SETTINGS --------------------------------------------------------------------
sub GetSetting { my ($k) = @_; return $settings{$k}; }
sub SetSetting { my ($k,$v) = @_; $settings{$k}=$v; }
## ENCODING STATUS
sub SetStatus
{
	my ($stat) = @_;
	$settings{'status'} = $stat;
}
sub GetStatus
{
	return $settings{'status'};
}
## BATCHSTATUS
sub SetBatchStatus
{
	my ($stat) = @_;
	$settings{'batchstatus'} = $stat;
}
sub GetBatchStatus
{
	return $settings{'batchstatus'};
}
## MENCODER PID
sub SetMencoderPID
{
	my ($pid) = @_;
	$settings{'$mencoderpid'} = $pid;
}
sub GetMencoderPID
{
	return $settings{'$mencoderpid'};
}

1;
