#!/usr/bin/perl -w

##  Copyright 2003 Rien Croonenborghs
##
##    This file is part of file2divx3pass.
##    file2divx4pass is free software; you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation; either version 2 of the License, or
##    (at your option) any later version.
##    file2divx3pass is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##    You should have received a copy of the GNU General Public License
##    along with file2divx3pass; if not, write to the Free Software
##    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package File2DiVX3Pass;
use strict;

use File2DiVX3Pass::helpers;
use File2DiVX3Pass::interface;

my $true  = 1;
my $false = 0;


## close the main window
sub wdwMainClose
{
	my ($wdw) = @_;
	$wdw->destroy();
	Gtk->exit( $false );
}


## SELECT INPUT FILE -----------------------------------------------------------

## click select file for input
sub btnInputClicked
{
	my ($button,$widgets) = @_;

	## create a file selection widgets + add signals
	$widgets->{'fsInput'} = new Gtk::FileSelection( "Select file to encode" );
	$widgets->{'fsInput'}->signal_connect( "delete_event", \&fsInputClose );
	$widgets->{'fsInput'}->ok_button->signal_connect( "clicked", \&fsInputOkButtonClicked, $widgets );
	$widgets->{'fsInput'}->cancel_button->signal_connect( "clicked", \&fsInputCancelButtonClicked, $widgets );
	$widgets->{'fsInput'}->show;
}
## close the file selection widgets for input files
sub fsInputClose
{
	my ($fsel) = @_;
	$fsel->destroy();
}
## cancel clicked for file selection for input files
sub fsInputCancelButtonClicked
{
	my ($button,$widgets) = @_;
	$widgets->{'fsInput'}->destroy();
}
## ok clicked for file selection for input files
sub fsInputOkButtonClicked
{
	my ($button,$widgets) = @_;
	## get filename
	$widgets->{'fsInput'}->hide();
	my $txt = $widgets->{fsInput}->get_filename();
	$widgets->{'fsInput'}->destroy();

	SetInputFile( $txt,$widgets );
}

## read the specs from the input file, and set the widgets
sub SetInputFile
{
	my ($filename,$widgets) = @_;

	MyPrint( $widgets, "Checking file..." );
	## get the specs of the selected files
	if( !ReadFileSpecs( $filename ) )
	{
		MyPrint( $widgets, GetError() );
		return;
	}

	## check the size of the CD
	my $cdsize = 0;
	if( !CheckNumeralEntry($widgets->{'entrySize'}) )
	{
		MyPrint( $widgets, GetError()." for CD size" );
		return;
	}
	else { $cdsize = $widgets->{'entrySize'}->get_text; }
	## check the audio bitrate
	my $audiobrate = 0;
	if( !CheckNumeralEntry( $widgets->{'cmbAudioBrate'}->entry ) )
	{
		MyPrint( $widgets, GetError()." for Audio MP3 bitrate" );
		return;
	}
	else { $audiobrate = $widgets->{'cmbAudioBrate'}->entry->get_text; }

	## put the filename in widget entryInput
	$widgets->{'entryInput'}->set_text( $filename );
	## put the filename in widget entryOutput
	## remove path and extension -> put avi instead
	my $f = substr( $filename, rindex($filename,"/")+1 );
	$f = substr( $f, 0, rindex($f,".") ).".avi";
	$widgets->{'entryOutput'}->set_text( $f );

	## put time in widget entryLength
	my $sec = GetSpecSeconds();
	## put the length in widget entryLength
	$widgets->{'entryLength'}->set_text( StoH($sec) );

	## calculate video bitrate
	my $fps = GetSpecFPS();
	my $videobrate = CalcVBrate( $cdsize, $sec/60, $audiobrate );
	## put the video brate in widget entryVideoBrate
	$widgets->{'entryVideoBrate'}->set_text( $videobrate );

	## set output FPS to spec FPS, even if the checkbutton is not active
	$widgets->{'entryOutputFPS'}->set_text( $fps );
}

## Audio ID checkbutton toggled
sub chkbtnAudioIDToggled
{
	my ($chkbtn, $widgets) = @_;
	MyPrint( $widgets, "" );
	if( $chkbtn->active )
	{
		$widgets->{'entryAudioID'}->set_sensitive( $true );
	}
	else
	{
		$widgets->{'entryAudioID'}->set_sensitive( $false );
	}
}

## Subtitle ID checkbutton toggled
sub chkbtnSubIDToggled
{
	my ($chkbtn, $widgets) = @_;
	MyPrint( $widgets, "" );
	if( $chkbtn->active )
	{
		$widgets->{'entrySubID'}->set_sensitive( $true );
	}
	else
	{
		$widgets->{'entrySubID'}->set_sensitive( $false );
	}
}

## set the subtitle file
sub btnSubFileClicked
{
	my ($button,$widgets) = @_;

	## create a file selection widgets + add signals
	$widgets->{'fsSubFile'} = new Gtk::FileSelection( "Select a subtitle file" );
	$widgets->{'fsSubFile'}->signal_connect( "delete_event", \&fsSubFileClose );
	$widgets->{'fsSubFile'}->ok_button->signal_connect( "clicked", \&fsSubFileOkButtonClicked, $widgets );
	$widgets->{'fsSubFile'}->cancel_button->signal_connect( "clicked", \&fsSubFileCancelButtonClicked, $widgets );
	$widgets->{'fsSubFile'}->show;
}
## close the file selection widgets for subtitle file
sub fsSubFileClose
{
	my ($fsel) = @_;
	$fsel->destroy();
}
## cancel clicked for file selection for subtitle file
sub fsSubFileCancelButtonClicked
{
	my ($button,$widgets) = @_;
	$widgets->{'fsSubFile'}->destroy();
}
## ok clicked for file selection for subtitle file
sub fsSubFileOkButtonClicked
{
	my ($button,$widgets) = @_;
	## get filename
	$widgets->{'fsSubFile'}->hide();
	my $txt = $widgets->{fsSubFile}->get_filename();
	$widgets->{'fsSubFile'}->destroy();

	##
	$widgets->{'entrySubFile'}->set_text( $txt );
}


sub chkbtnSubFileToggled
{
	my ($chkbtn,$widgets) = @_;

	if( !$widgets->{'chkbtnSubFile'}->active() )
	{
		$widgets->{'entrySubFile'}->set_sensitive( $false );
		$widgets->{'btnSubFile'}->set_sensitive( $false );
	}
	else
	{
		$widgets->{'entrySubFile'}->set_sensitive( $true );
		$widgets->{'btnSubFile'}->set_sensitive( $true );
	}
}

## SETTINGS CHANGED ------------------------------------------------------------

## size of CD is changed
sub entrySizeActivated
{
	my ($entry,$widgets) = @_;
	MyPrint( $widgets, "" );

	if( $widgets->{'entryInput'}->get_text ne "" )
	{
		## check size of the CD
		my $cdsize = 0;
		if( !CheckNumeralEntry($widgets->{'entrySize'}) )
		{
			MyPrint( $widgets, GetError()." for CD size" );
			return;
		}
		else { $cdsize = $widgets->{'entrySize'}->get_text; }
		## check audio bitrate
		my $audiobrate = 0;
		if( !CheckNumeralEntry( $widgets->{'cmbAudioBrate'}->entry ) )
		{
			MyPrint( $widgets, GetError()." for Audio MP3 bitrate" );
			return;
		}
		else { $audiobrate = $widgets->{'cmbAudioBrate'}->entry->get_text; }
		## check length of the file hh:mm:ss
		my $min = 1;
		if( !CheckHourEntry( $widgets->{'entryLength'} ) )
		{
			MyPrint( $widgets, GetError()." for File length" );
			return;
		}
		else { $min = HtoS( $widgets->{'entryLength'}->get_text )/60; }

		## calculate video bitrate
		my $videobrate = CalcVBrate( $cdsize, $min, $audiobrate );
		## put the video brate in widget entryVideoBrate
		$widgets->{'entryVideoBrate'}->set_text( $videobrate );
	}
}

## length of file is changed
sub entryLengthActivated
{
	my ($entry,$widgets) = @_;
	entrySizeActivated( undef, $widgets );
}

## audio codec has changed
sub cmbAudioCodecChanged
{
	my ($entry,$widgets) = @_;

	if( $entry->get_text() eq "mp3" )
	{
		$widgets->{'lblAudioBrate'}->set_sensitive( $true );
		$widgets->{'cmbAudioBrate'}->set_sensitive( $true );
		$widgets->{'lblAudioRate'}->set_sensitive( $true );
		$widgets->{'cmbAudioRate'}->set_sensitive( $true );
	}
	elsif( $entry->get_text() eq "copy" )
	{
		$widgets->{'lblAudioBrate'}->set_sensitive( $false );
		$widgets->{'cmbAudioBrate'}->set_sensitive( $false );
		$widgets->{'lblAudioRate'}->set_sensitive( $false );
		$widgets->{'cmbAudioRate'}->set_sensitive( $false );
	}
	elsif( $entry->get_text() eq "pcm" )
	{
		$widgets->{'lblAudioBrate'}->set_sensitive( $false );
		$widgets->{'cmbAudioBrate'}->set_sensitive( $false );
		$widgets->{'lblAudioRate'}->set_sensitive( $false );
		$widgets->{'cmbAudioRate'}->set_sensitive( $false );
	}
}

## audio bitrate has changed
sub cmbAudioBrateEntryChanged
{
	my ($entry,$widgets) = @_;
	entrySizeActivated( undef, $widgets );
}

## video bitrate has changed
sub entryVideoBrateChanged
{
	my ($entry,$widgets) = @_;
	MyPrint( $widgets, "" );

	if( $widgets->{'entryInput'}->get_text ne "" )
	{
		## check output fps
		## from spec or checked from the entry
		my $fps = 0;
		if( $widgets->{'chkbtnOutputFPS'}->active )
		{ $fps = $widgets->{'entryOutputFPS'}->get_text; }
		else { $fps = GetSpecFPS(); }
		## check scale width
		my $width = 0;
		if( !CheckNumeralEntry( $widgets->{'entryScaleWidth'} ) )
		{
			MyPrint( $widgets, GetError()." for Scale width" );
			return;
		}
		else { $width = $widgets->{'entryScaleWidth'}->get_text; }
		## check scale height
		my $height = 0;
		if( !CheckNumeralEntry( $widgets->{'entryScaleHeight'} ) )
		{
			MyPrint( $widgets, GetError()." for Scale Height" );
			return;
		}
		else { $height = $widgets->{'entryScaleHeight'}->get_text; }
		## check video bitrate
		my $videobrate = 0;
		if( !CheckNumeralEntry( $widgets->{'entryVideoBrate'} ) )
		{
			MyPrint( $widgets, GetError()." for Video bitrate" );
			return;
		}
		else { $videobrate = $widgets->{'entryVideoBrate'}->get_text; }

		MyPrint( $widgets, "" );
		## calculate bpp
		my $bpp = CalcBPP( $videobrate, $width, $height, $fps );
		$bpp = substr( $bpp, 0, index($bpp,".")+4);
		## put bpp in widget entryVideoBPP
		$widgets->{'entryVideoBPP'}->set_text( $bpp );

	}
}

## scale width has changed
sub entryScaleWidthActivated
{
	my ($entry, $widgets) = @_;

	my $sch = 0;
	my $scw = 0;

	## calc weither it's 4:3 or 16:9
	MyPrint( $widgets, "" );

	if( $widgets->{'rbtnScale43'}->active )
	{
  		$sch = 4;
		$scw = 3;
	}
	elsif( $widgets->{'rbtnScale169'}->active )
	{
 		$sch = 16;
		$scw = 9;
	}

	## check scale width
	my $w = 0;
	if( !CheckNumeralEntry( $widgets->{'entryScaleWidth'} ) )
	{
		MyPrint( $widgets, GetError()." for Scale width" );
		return;
	}
	else { $w = $widgets->{'entryScaleWidth'}->get_text; }

	my $h = ($w/$sch)*$scw;

	entrySizeActivated( undef, $widgets );
	$widgets->{'entryScaleHeight'}->set_text( $h );
	entryVideoBrateChanged( undef, $widgets );
}

## scale height has changed
sub entryScaleHeightActivated
{
	my ($entry, $widgets) = @_;

	my $sch = 0;
	my $scw = 0;

	## calc weither it's 4:3 or 16:9
	MyPrint( $widgets, "" );

	if( $widgets->{'rbtnScale43'}->active )
	{
  		$sch = 4;
		$scw = 3;
	}
	elsif( $widgets->{'rbtnScale169'}->active )
	{
 		$sch = 16;
		$scw = 9;
	}

	## check scale height
	my $h = 0;
	if( !CheckNumeralEntry( $widgets->{'entryScaleHeight'} ) )
	{
		MyPrint( $widgets, GetError()." for Scale height" );
		return;
	}
	else { $h = $widgets->{'entryScaleHeight'}->get_text; }

	my $w = ($h/$scw)*$sch;

	entrySizeActivated( undef, $widgets );
	$widgets->{'entryScaleWidth'}->set_text( $w );
	entryVideoBrateChanged( undef, $widgets );
}

## 4:3 has toggled
sub rbtnScale43Toggled
{
	my ($rbtn,$widgets) = @_;
	MyPrint( $widgets, "" );
	entryScaleWidthActivated( undef, $widgets );
}


## OutputFPS checkbutton toggled
sub chkbtnOutputFPSToggled
{
	my ($chkbtn, $widgets) = @_;
	MyPrint( $widgets, "" );
	if( $chkbtn->active )
	{
		$widgets->{'entryOutputFPS'}->set_sensitive( $true );
	}
	else
	{
		$widgets->{'entryOutputFPS'}->set_sensitive( $false );
	}
	entrySizeActivated( undef, $widgets );
	entryVideoBrateChanged( undef, $widgets );
}

## OutputFPS entry activated
sub entryOutputFPSActivated
{
	my ($entry,$widgets) = @_;
	MyPrint( $widgets, "" );
	if( $widgets->{'chkbtnOutputFPS'}->active )
	{
		entrySizeActivated( undef, $widgets );
		entryVideoBrateChanged( undef, $widgets );
	}
}

## codec changed
## change it's options
sub cmbCodecEntryChanged
{
	my ($cmd,$widgets) = @_;
	MyPrint( $widgets, "" );
	if( $widgets->{'cmbCodec'}->entry->get_text eq "divx4" )
	{
		$widgets->{'entryLength'}->set_sensitive( $true );
		$widgets->{'entrySize'}->set_sensitive( $true );
		$widgets->{'chkbtnOutputFPS'}->set_sensitive( $true );
		$widgets->{'entryOutputFPS'}->set_sensitive( $true ) if( $widgets->{'chkbtnOutputFPS'}->active() );
		$widgets->{'entryScaleWidth'}->set_sensitive( $true );
		$widgets->{'entryScaleHeight'}->set_sensitive( $true );
		$widgets->{'rbtnScale43'}->set_sensitive( $true );
		$widgets->{'rbtnScale169'}->set_sensitive( $true );
		$widgets->{'entryVideoBrate'}->set_sensitive( $true );
		$widgets->{'entryVideoBPP'}->set_sensitive( $true );
		$widgets->{'cmbPass'}->set_sensitive( $true );
		$widgets->{'entryEncodingOpts'}->set_sensitive( $true );

		$widgets->{'entryEncodingOpts'}->set_text( GetSetting( "divx4extra" ) );
	}
	elsif( $widgets->{'cmbCodec'}->entry->get_text eq "lavc" )
	{
		$widgets->{'entryLength'}->set_sensitive( $true );
		$widgets->{'entrySize'}->set_sensitive( $true );
		$widgets->{'chkbtnOutputFPS'}->set_sensitive( $true );
		$widgets->{'entryOutputFPS'}->set_sensitive( $true ) if( $widgets->{'chkbtnOutputFPS'}->active() );
		$widgets->{'entryScaleWidth'}->set_sensitive( $true );
		$widgets->{'entryScaleHeight'}->set_sensitive( $true );
		$widgets->{'rbtnScale43'}->set_sensitive( $true );
		$widgets->{'rbtnScale169'}->set_sensitive( $true );
		$widgets->{'entryVideoBrate'}->set_sensitive( $true );
		$widgets->{'entryVideoBPP'}->set_sensitive( $true );
		$widgets->{'cmbPass'}->set_sensitive( $true );
		$widgets->{'entryEncodingOpts'}->set_sensitive( $true );

		$widgets->{'entryEncodingOpts'}->set_text( GetSetting( "lavcextra" ) );
	}
	elsif( $widgets->{'cmbCodec'}->entry->get_text eq "copy" )
	{
		my $vb = GetSpecVBitrate()/1000;
		$vb = substr($vb,0,index($vb,"."));

		if( $vb eq "" ) { $vb = 1000; }

		$widgets->{'entryVideoBrate'}->set_text( $vb );

		$widgets->{'entryLength'}->set_sensitive( $false );
		$widgets->{'entrySize'}->set_sensitive( $false );
		$widgets->{'chkbtnOutputFPS'}->set_sensitive( $false );
		$widgets->{'entryOutputFPS'}->set_sensitive( $false );
		$widgets->{'entryScaleWidth'}->set_sensitive( $false );
		$widgets->{'entryScaleHeight'}->set_sensitive( $false );
		$widgets->{'rbtnScale43'}->set_sensitive( $false );
		$widgets->{'rbtnScale169'}->set_sensitive( $false );
		$widgets->{'entryVideoBrate'}->set_sensitive( $false );
		$widgets->{'entryVideoBPP'}->set_sensitive( $false );
		$widgets->{'cmbPass'}->set_sensitive( $false );
		$widgets->{'entryEncodingOpts'}->set_sensitive( $false );
	}
}

## if encoding opts change, put the new in settings
sub entryEncodingOptsChanged
{
	my ($entry,$widgets) = @_;
	SetSetting( $widgets->{'cmbCodec'}->entry->get_text."extra", $widgets->{'entryEncodingOpts'}->get_text );

}

## START - STOP ----------------------------------------------------------------

## button start is clicked
sub btnStartClicked
{
	my ($btn, $widgets) = @_;

	MyPrint( $widgets, "" );
	if( CheckNonNumeralEntry ( $widgets->{'entryInput'} ) )
	{
		## check subtitle file
		my $subfile = -1;
		if( $widgets->{'chkbtnSubFile'}->active() )
		{
			if( !CheckNonNumeralEntry ( $widgets->{'entrySubFile'} ) )
			{
				MyPrint( $widgets, GetError()." for subtitle file" );
			}
			else { $subfile = $widgets->{'entrySubFile'}->get_text; }
		}
		## check output
		my $outfile = "out.divx.avi";
		if( !CheckNonNumeralEntry ( $widgets->{'entryOutput'} ) )
		{
			MyPrint( $widgets, GetError()." for Output file" );
		}
		else { $outfile = $widgets->{'entryOutput'}->get_text; }
		## check audio codec
		my $audiocodec = "mp3";
		if( !CheckNonNumeralEntry( $widgets->{'cmbAudioCodec'}->entry ) )
		{
			MyPrint( $widgets, GetError()." for Audio Codec" );
			return;
		}
		else { $audiocodec = $widgets->{'cmbAudioCodec'}->entry->get_text; }
		## check audio bitrate
		my $audiobrate = 0;
		if( !CheckNumeralEntry( $widgets->{'cmbAudioBrate'}->entry ) )
		{
			MyPrint( $widgets, GetError()." for Audio MP3 bitrate" );
			return;
		}
		else { $audiobrate = $widgets->{'cmbAudioBrate'}->entry->get_text; }
		## check audio rate
		my $audiorate = 0;
		if( !CheckNonNumeralEntry( $widgets->{'cmbAudioRate'}->entry ) )
		{
			MyPrint( $widgets, GetError()." for Bitrate" );
			return;
		}
		else { $audiorate = $widgets->{'cmbAudioRate'}->entry->get_text; }
		## check scale width
		my $width = 0;
		if( !CheckNumeralEntry( $widgets->{'entryScaleWidth'} ) )
		{
			MyPrint( $widgets, GetError()." for Scale width" );
			return;
		}
		else { $width = $widgets->{'entryScaleWidth'}->get_text; }
		## check scale height
		my $height = 0;
		if( !CheckNumeralEntry( $widgets->{'entryScaleHeight'} ) )
		{
			MyPrint( $widgets, GetError()." for Scale height" );
			return;
		}
		else { $height = $widgets->{'entryScaleHeight'}->get_text; }
		## check video bitrate
		my $videobrate = 0;
		if( !CheckNumeralEntry( $widgets->{'entryVideoBrate'} ) )
		{
			MyPrint( $widgets, GetError()." for Video bitrate" );
			return;
		}
		else { $videobrate = $widgets->{'entryVideoBrate'}->get_text; }
		## check video codec
		my $videocodec = 0;
		if( !CheckNonNumeralEntry( $widgets->{'cmbCodec'}->entry ) )
		{
			MyPrint( $widgets, GetError()." for Video codec" );
			return;
		}
		else { $videocodec = $widgets->{'cmbCodec'}->entry->get_text; }
		## check pass
		my $pass = 0;
		if( !CheckNonNumeralEntry( $widgets->{'cmbPass'}->entry ) )
		{
			MyPrint( $widgets, GetError()." for Pass" );
			return;
		}
		else { $pass = $widgets->{'cmbPass'}->entry->get_text; }
		## encoding options
		my $encopts = "";
		if( !CheckNonNumeralEntry( $widgets->{'entryEncodingOpts'} ) )
		{
			MyPrint( $widgets, GetError()." for Pass" );
			return;
		}
		else { $encopts = $widgets->{'entryEncodingOpts'}->get_text; }
		## check ofps
		my $ofps = -1;
		if( $widgets->{'chkbtnOutputFPS'}->active )
		{
			if( !CheckNonNumeralEntry( $widgets->{'entryOutputFPS'} ) )
			{
				MyPrint( $widgets, GetError()." for Output FPS" );
				return;
			}
			else { $ofps = $widgets->{'entryOutputFPS'}->get_text; }
		}
		## audio id
		my $audioid = -1;
		if( $widgets->{'chkbtnAudioID'}->active )
		{
			if( !CheckNumeralEntry( $widgets->{'entryAudioID'} ) )
			{
				MyPrint( $widgets, GetError()." for Audio ID" );
				return;
			}
			else { $audioid = $widgets->{'entryAudioID'}->get_text; }
		}
		## subtitle id
		my $subid = -1;
		if( $widgets->{'chkbtnSubID'}->active )
		{
			if( !CheckNumeralEntry( $widgets->{'entrySubID'} ) )
			{
				MyPrint( $widgets, GetError()." for Subtitle ID" );
				return;
			}
			else { $subid = $widgets->{'entrySubID'}->get_text; }
		}
		## input file
		my $infile = $widgets->{'entryInput'}->get_text;

		## start the encoding if not already started
		if( GetStatus() ne "started" )
		{
			SetStatus( "started" );
			$widgets->{'btnStart'}->set_sensitive( $false );
			$widgets->{'btnStop'}->set_sensitive( $true );
			while (Gtk->events_pending()) { Gtk->main_iteration() };
			if( EncodeVideo( $infile, $subfile, $subid, $audioid, $outfile, $audiocodec, $audiobrate, $audiorate, $width, $height, $videobrate, $videocodec, $pass, $encopts, $ofps, $widgets ) )
			{
				$widgets->{'btnStart'}->set_sensitive( $true );
				$widgets->{'btnStop'}->set_sensitive( $false );
				MyPrint( $widgets, "Done" );
			}
			else
			{
				MyPrint( $widgets, GetError() );
			}
			SetStatus( "stopped" );
		}
		else { MyPrint( $widgets, "Already busy !!" ) };
	}
}

## button stop is clicked
sub btnStopClicked
{
	my ($button,$widgets) = @_;

	if( GetStatus() eq "started" )
	{
		my $pid = GetMencoderPID();
		if( defined( $pid ) )
		{
			## kill the menoder PID
			## release buttons
			++$pid;
			kill 9, $pid;
			$widgets->{'btnStart'}->set_sensitive( $true );
			$widgets->{'btnStop'}->set_sensitive( $false );
			SetStatus( "stopped" );
		}
	}
}


## MENU ------------------------------------------------------------------------

## click on menu help>about
sub MenuAbout
{
	my ($version) = @_;

	my $wdw = new Gtk::Window( "toplevel" );
	$wdw->set_title( "About" );
	$wdw->set_policy( $false, $false, $true );
	$wdw->set_position( "center" );
	$wdw->border_width( 5 );
	$wdw->set_modal( $true );

	my $frm = new Gtk::Frame();
	$frm->border_width(3);
	$wdw->add( $frm );
	my $vb = new Gtk::VBox( $false, 5 );
	$vb->border_width(5);
	$frm->add( $vb );

	my $lbl1 = new Gtk::Label( "  File2DiVX3Pass $version  \n  by Rien (rien\@geekshop.be)  " );
	my $sep1 = new Gtk::HSeparator();
	my $lbl2 = new Gtk::Label( "  Because an idle CPU is a terrible thing to waste  " );
	my $sep2 = new Gtk::HSeparator();
	my $btn = new Gtk::Button( "   close   " );
	Add2BoxStart( $vb, $lbl1 );
	Add2BoxStart( $vb, $sep1 );
	Add2BoxStart( $vb, $lbl2 );
	Add2BoxStart( $vb, $sep2 );
	Add2BoxStart( $vb, $btn );

	$wdw->signal_connect( "delete_event", sub{ my ($wdw) = @_; $wdw->destroy(); } );
	$btn->signal_connect( "clicked", sub{ my ($btn,$wdw) = @_; $wdw->destroy(); }, $wdw );

	$wdw->show_all;

}

## click on menu new
sub MenuNew
{
	my ($widgets) = @_;

	if( GetStatus() ne "started" )
	{
		## clear input, sub and output
		$widgets->{'entryInput'}->set_text( "" );
		$widgets->{'entrySubFile'}->set_text( "" );
		$widgets->{'entrySubFile'}->set_sensitive( $false );
		$widgets->{'btnSubFile'}->set_sensitive( $false );
		$widgets->{'chkbtnSubFile'}->set_active( $false );
		$widgets->{'entryAudioID'}->set_text( "" );
		$widgets->{'entryAudioID'}->set_sensitive( $false );
		$widgets->{'chkbtnAudioID'}->set_active( $false );
		$widgets->{'entrySubID'}->set_text( "" );
		$widgets->{'entrySubID'}->set_sensitive( $false );
		$widgets->{'chkbtnSubID'}->set_active( $false );
		$widgets->{'entryOutput'}->set_text( "out.divx.avi" );

		## reset audio bitrate and rate
		$widgets->{'cmbAudioCodec'}->list->select_item( 1 );
		$widgets->{'cmbAudioBrate'}->list->select_item( 8 );
		$widgets->{'cmbAudioRate'}->list->select_item( 1 );

		## clear file length
		$widgets->{'entryLength'}->set_text( StoH(0) );
		## reset cd size
		$widgets->{'entrySize'}->set_text( "700" );
		## reset scaling
		$widgets->{'entryScaleWidth'}->set_text( "480" );
		$widgets->{'entryScaleHeight'}->set_text( "360" );
		## reset aspect ratio
		$widgets->{'rbtnScale43'}->active;
		## clear video bitrate
		$widgets->{'entryVideoBrate'}->set_text( "" );
		$widgets->{'entryVideoBPP'}->set_text( "" );

		## reset codec
		$widgets->{'cmbCodec'}->list->select_item( 0 );
		## reset pass
		$widgets->{'cmbPass'}->list->select_item( 2 );
		## reset ecoding options
		if( $widgets->{'cmbCodec'}->entry->get_text eq "divx4" )
		{
			$widgets->{'entryEncodingOpts'}->set_text( GetSetting( "divx4extra" ) );
		}
		elsif( $widgets->{'cmbCodec'}->entry->get_text eq "lavc" )
		{
			$widgets->{'entryEncodingOpts'}->set_text( GetSetting( "lavcextra" ) );
		}

		## reset output FPS
		$widgets->{'chkbtnOutputFPS'}->set_active( $false );
		$widgets->{'entryOutputFPS'}->set_sensitive( $false );
		$widgets->{'entryOutputFPS'}->set_text( "25" );

	}
}

## click on menu save
sub MenuSave
{
	my ($widgets) = @_;

	if( GetStatus() ne "started" )
	{
		## start with a file selection where to save to
		$widgets->{'fsSave'} = new Gtk::FileSelection( "Save project" );
		$widgets->{'fsSave'}->signal_connect( "delete_event", \&fsSaveClose );
		$widgets->{'fsSave'}->ok_button->signal_connect( "clicked", \&fsSaveOkButtonClicked, $widgets );
		$widgets->{'fsSave'}->cancel_button->signal_connect( "clicked", \&fsSaveCancelButtonClicked, $widgets );
		$widgets->{'fsSave'}->show;
	}
}
## close the file selection for saving the project
sub fsSaveClose
{
	my ($fsel) = @_;
	$fsel->destroy;
}
## cancel saving the project
sub fsSaveCancelButtonClicked
{
	my ($button,$widgets) = @_;
	$widgets->{'fsSave'}->destroy;
}
## save the project
sub fsSaveOkButtonClicked
{
	my ($button,$widgets) = @_;
	$widgets->{'fsSave'}->hide;
	my $txt = $widgets->{'fsSave'}->get_filename;
	$widgets->{'fsSave'}->destroy;

	my $res = open(FP,">$txt");
	if( !defined($res) )
	{
		$widgets->{'lblActions'}->set_text( "Could not save project" );
		return;
	}

	print FP "entryInput=".$widgets->{'entryInput'}->get_text()."\n";
	if( $widgets->{'chkbtnSubFile'}->active )
	{ print FP "chkbtnSubFile=true\n"; }
	print FP "entrySubFile=".$widgets->{'entrySubFile'}->get_text()."\n";
	if( $widgets->{'chkbtnAudioID'}->active )
	{ print FP "chkbtnAudioID=true\n"; }
	print FP "entryAudioID=".$widgets->{'entryAudioID'}->get_text()."\n";
	if( $widgets->{'chkbtnSubID'}->active )
	{ print FP "chkbtnSubID=true\n"; }
	print FP "entrySubID=".$widgets->{'entrySubID'}->get_text()."\n";
	print FP "entryLength=".$widgets->{'entryLength'}->get_text()."\n";
	print FP "entryVideoBrate=".$widgets->{'entryVideoBrate'}->get_text()."\n";
	print FP "entryVideoBPP=".$widgets->{'entryVideoBPP'}->get_text()."\n";
	print FP "entryOutput=".$widgets->{'entryOutput'}->get_text()."\n";
	print FP "cmbAudioCodec=".$widgets->{'cmbAudioCodec'}->entry->get_text()."\n";
	print FP "cmbAudioBrate=".$widgets->{'cmbAudioBrate'}->entry->get_text()."\n";
	print FP "cmbAudioRate=".$widgets->{'cmbAudioRate'}->entry->get_text()."\n";
	print FP "entrySize=".$widgets->{'entrySize'}->get_text()."\n";
	print FP "entryScaleWidth=".$widgets->{'entryScaleWidth'}->get_text()."\n";
	print FP "entryScaleHeight=".$widgets->{'entryScaleHeight'}->get_text()."\n";
	if( $widgets->{'rbtnScale43'}->active )
	{ print FP "rbtnScale43=true\n"; }
	else { print FP "rbtnScale43=false\n"; }
	print FP "cmbCodec=".$widgets->{'cmbCodec'}->entry->get_text()."\n";
	print FP "cmbPass=".$widgets->{'cmbPass'}->entry->get_text()."\n";
	if( $widgets->{'chkbtnOutputFPS'}->active )
	{ print FP "chkbtnOutputFPS=true\n"; }
	print FP "entryOutputFPS=".$widgets->{'entryOutputFPS'}->get_text()."\n";
	close( FP );

	$widgets->{'lblActions'}->set_text( "Project saved" );
}

## click menu open
sub MenuOpen
{
	my ($widgets) = @_;

	if( GetStatus() ne "started" )
	{
		## file selection for opening the project
		$widgets->{'fsOpen'} = new Gtk::FileSelection( "Open project" );
		$widgets->{'fsOpen'}->signal_connect( "delete_event", \&fsOpenClose );
		$widgets->{'fsOpen'}->ok_button->signal_connect( "clicked", \&fsOpenOkButtonClicked, $widgets );
		$widgets->{'fsOpen'}->cancel_button->signal_connect( "clicked", \&fsOpenCancelButtonClicked, $widgets );
		$widgets->{'fsOpen'}->show;
	}
}
## close the file selection for opening project
sub fsOpenClose
{
	my ($fsel) = @_;
	$fsel->destroy;
}
## cancel opening the project
sub fsOpenCancelButtonClicked
{
	my ($button,$widgets) = @_;
	$widgets->{'fsOpen'}->destroy;
}
## open project
sub fsOpenOkButtonClicked
{
	my ($button,$widgets) = @_;
	$widgets->{'fsOpen'}->hide;
	my $txt = $widgets->{'fsOpen'}->get_filename;
	$widgets->{'fsOpen'}->destroy;

	ReadProject( $txt, $widgets );

	$widgets->{'lblActions'}->set_text( "Project opened" );
}
## read project file and put in widgets
sub ReadProject
{
	my ($filename,$widgets) = @_;
	my $res = open(FP,"<$filename");
	if( !defined($res) )
	{
		$widgets->{'lblActions'}->set_text( "Could not open project $filename" );
		return;
	}
	while( <FP> )
	{
		my ($k,$v) = split( "=", $_ );
		chop( $v );

		if( $k eq "entryInput" )
		{ $widgets->{'entryInput'}->set_text( $v ); ReadFileSpecs( $v ); }
		if( $k eq "chkbtnSubFile" && $v eq "true" )
		{ $widgets->{'chkbtnSubFile'}->set_active( $true ); $widgets->{'entrySubFile'}->set_sensitive( $true ); }
		if( $k eq "entrySubFile" )
		{ $widgets->{'entrySubFile'}->set_text( $v ); }
		if( $k eq "chkbtnAudioID" && $v eq "true" )
		{ $widgets->{'chkbtnAudioID'}->set_active( $true ); $widgets->{'entryAudioID'}->set_sensitive( $true ); }
		if( $k eq "entryAudioID" )
		{ $widgets->{'entryAudioID'}->set_text( $v ); }
		if( $k eq "chkbtnSubID" && $v eq "true" )
		{ $widgets->{'chkbtnSubID'}->set_active( $true ); $widgets->{'entrySubID'}->set_sensitive( $true ); }
		if( $k eq "entrySubID" )
		{ $widgets->{'entrySubID'}->set_text( $v ); }
		if( $k eq "entryLength" )
		{ $widgets->{'entryLength'}->set_text( $v ); }
		if( $k eq "entryVideoBrate" )
		{ $widgets->{'entryVideoBrate'}->set_text( $v ); }
		if( $k eq "entryVideoBPP" )
		{ $widgets->{'entryVideoBPP'}->set_text( $v ); }
		if( $k eq "entryOutput" )
		{ $widgets->{'entryOutput'}->set_text( $v ); }
		if( $k eq "cmbAudioCodec" )
		{ $widgets->{'cmbAudioCodec'}->entry->set_text( $v ); }
		if( $k eq "cmbAudioBrate" )
		{ $widgets->{'cmbAudioBrate'}->entry->set_text( $v ); }
		if( $k eq "cmbAudioRate" )
		{ $widgets->{'cmbAudioRate'}->entry->set_text( $v ); }
		if( $k eq "entrySize" )
		{ $widgets->{'entrySize'}->set_text( $v ); }
		if( $k eq "entryScaleWidth" )
		{ $widgets->{'entryScaleWidth'}->set_text( $v ); }
		if( $k eq "entryScaleHeight" )
		{ $widgets->{'entryScaleHeight'}->set_text( $v ); }
		if( $k eq "rbtnScale43" && $v eq "true" )
		{ $widgets->{'rbtnScale43'}->active; }
		if( $k eq "cmbCodec" )
		{ $widgets->{'cmbCodec'}->entry->set_text( $v ); }
		if( $k eq "cmbPass" )
		{ $widgets->{'cmbPass'}->entry->set_text( $v ); }
		if( $k eq "chkbtnOutputFPS" && $v eq "true" )
		{ $widgets->{'chkbtnOutputFPS'}->set_active( $true ); $widgets->{'entryOutputFPS'}->set_sensitive( $true ); }
		if( $k eq "entryOutputFPS" )
		{ $widgets->{'entryOutputFPS'}->set_text( $v ); }
	}
	close( FP );
}

## BATCH ENCODING --------------------------------------------------------------

## click menu batch encode
sub MenuBatchEncode
{
	my ($widgets) = @_;

	if( GetStatus() ne "started" )
	{
		## show window for batch encoding
		## if this widgets does NOT exist
		## else show it
		if( !defined($widgets->{'wdwBatchMain'}) )
		{
			$widgets->{'wdwBatchMain'} = new Gtk::Window( "toplevel" );
			$widgets->{'wdwBatchMain'}->set_title( "Batch encode" );
			$widgets->{'wdwBatchMain'}->set_policy( $false, $false, $true );
			$widgets->{'wdwBatchMain'}->set_position( "center" );
			$widgets->{'wdwBatchMain'}->border_width( 5 );

			$widgets->{'vbBatchMain'} = new Gtk::VBox( $false, 2 );
			$widgets->{'vbBatchMain'}->border_width( 5 );
			$widgets->{'wdwBatchMain'}->add( $widgets->{'vbBatchMain'} );

			$widgets->{'frmBatchMain'} = new Gtk::Frame();
			Add2BoxFillStart( $widgets->{'vbBatchMain'}, $widgets->{'frmBatchMain'} );

			$widgets->{'hbBatchMain'} = new Gtk::HBox( $false, 2 );
			$widgets->{'hbBatchMain'}->border_width( 5 );
			$widgets->{'frmBatchMain'}->add( $widgets->{'hbBatchMain'} );

				$widgets->{'swBatchList'} = new Gtk::ScrolledWindow( undef, undef );
			Add2BoxFillStart( $widgets->{'hbBatchMain'}, $widgets->{'swBatchList'} );
				$widgets->{'swBatchList'}->set_policy( "automatic","automatic" );
				$widgets->{'lstBatchList'} = new Gtk::List();
				$widgets->{'swBatchList'}->add_with_viewport( $widgets->{'lstBatchList'} );
				$widgets->{'lstBatchList'}->set_selection_mode( "single" );
				$widgets->{'swBatchList'}->set_usize( 200, 200 );

				$widgets->{'vbBatchButtons'} = new Gtk::VBox( $false, 2 );
			Add2BoxStart( $widgets->{'hbBatchMain'}, $widgets->{'vbBatchButtons'} );
					$widgets->{'btnBatchAddProject'} = new Gtk::Button( "Add project..." );
					$widgets->{'btnBatchRemoveProject'} = new Gtk::Button( "Remove project" );
					$widgets->{'btnBatchRemoveProjects'} = new Gtk::Button( " Remove projects " );
					$widgets->{'btnBatchStop'} = new Gtk::Button( "STOP" );
					$widgets->{'btnBatchStart'} = new Gtk::Button( "START" );
				Add2BoxStart( $widgets->{'vbBatchButtons'}, $widgets->{'btnBatchAddProject'} );
				Add2BoxStart( $widgets->{'vbBatchButtons'}, $widgets->{'btnBatchRemoveProject'} );
				Add2BoxStart( $widgets->{'vbBatchButtons'}, $widgets->{'btnBatchRemoveProjects'} );
				Add2BoxEnd( $widgets->{'vbBatchButtons'}, $widgets->{'btnBatchStop'} );
				Add2BoxEnd( $widgets->{'vbBatchButtons'}, $widgets->{'btnBatchStart'} );



			$widgets->{'hbBatchActions'} = new Gtk::HBox( $false, 1 );
			Add2BoxStart( $widgets->{'vbBatchMain'}, $widgets->{'hbBatchActions'} );
				$widgets->{'lblBatchActions'} = new Gtk::Label( "" );
				$widgets->{'lblBatchActions'}->set_justify( "left" );
				Add2BoxStart( $widgets->{'hbBatchActions'}, $widgets->{'lblBatchActions'} );

			## signals
			$widgets->{'wdwBatchMain'}->signal_connect( "delete_event", \&wdwBatchMainClosed, $widgets );
			$widgets->{'btnBatchAddProject'}->signal_connect( "clicked", \&btnBatchAddProjectClicked, $widgets );
			$widgets->{'btnBatchRemoveProject'}->signal_connect( "clicked", \&btnBatchRemoveProjectClicked, $widgets );
			$widgets->{'btnBatchRemoveProjects'}->signal_connect( "clicked", \&btnBatchRemoveProjectsClicked, $widgets );
			$widgets->{'btnBatchStart'}->signal_connect( "clicked", \&btnBatchStartClicked, $widgets );
			$widgets->{'btnBatchStop'}->signal_connect( "clicked", \&btnBatchStopClicked, $widgets );

			$widgets->{'wdwBatchMain'}->show_all();
		}
		else
		{
			while (Gtk->events_pending()) {Gtk->main_iteration()};
			$widgets->{'wdwBatchMain'}->show_all;
			while (Gtk->events_pending()) {Gtk->main_iteration()};
		}
	}
	else
	{
		while (Gtk->events_pending()) {Gtk->main_iteration()};
		$widgets->{'wdwBatchMain'}->show_all();
		while (Gtk->events_pending()) {Gtk->main_iteration()};
	}
}

## close the batch encode window
## hide it is what we want ... if filled in, 2nd time use start batch encode
## the list will still be there
## bug here : no debug last line -> hiding the widgets will destroy it in some kind
sub wdwBatchMainClosed
{
	my ($wdw, $widgets) = @_;

	$widgets->{'wdwBatchMain'}->hide;
	Debug( "" );
}

## add project to list
sub btnBatchAddProjectClicked
{
	my ($button,$widgets) = @_;

	## start file selection
	## multiple selection !!
	$widgets->{'fsBatchAddProject'} = new Gtk::FileSelection( "Select project to add" );
	$widgets->{'fsBatchAddProject'}->signal_connect( "delete_event", \&fsBatchAddProjectClose );
	$widgets->{'fsBatchAddProject'}->ok_button->signal_connect( "clicked", \&fsBatchAddProjectOkButtonClicked, $widgets );
	$widgets->{'fsBatchAddProject'}->cancel_button->signal_connect( "clicked", \&fsBatchAddProjectCancelButtonClicked, $widgets );
	$widgets->{'fsBatchAddProject'}->file_list->set_selection_mode( "multiple" );
	$widgets->{'fsBatchAddProject'}->show;
}
## add project to list closed
sub fsBatchAddProjectClose
{
	my ($filesel) = @_;
	$filesel->destroy;
}
## cancel adding project to list
sub fsBatchAddProjectCancelButtonClicked
{
	my ($button,$widgets) = @_;
	$widgets->{'fsBatchAddProject'}->destroy;
}
## add project to list
sub fsBatchAddProjectOkButtonClicked
{
	my ($button,$widgets) = @_;
 	$widgets->{'fsBatchAddProject'}->hide;

	my @filenames = $widgets->{'fsBatchAddProject'}->file_list->selection;
	if( $#filenames > -1 )
	{
		for( my $i=0; $i<@filenames; ++$i )
		{
			my $txt = $widgets->{'fsBatchAddProject'}->selection_text->get;
			$txt = substr($txt,11);
			$txt .= "/".$widgets->{'fsBatchAddProject'}->file_list->get_text( $filenames[$i], 0 );
			my $item = new Gtk::ListItem( $txt );
			$item->show;
			$widgets->{'lstBatchList'}->add( $item );
		}
	}
	$widgets->{'fsBatchAddProject'}->destroy;
}

## remove selected project from list
sub btnBatchRemoveProjectClicked
{
	my ($button, $widgets) = @_;
	my @sel = $widgets->{'lstBatchList'}->selection;
	$widgets->{'lstBatchList'}->remove_items( @sel );
}
## remove all projects
sub btnBatchRemoveProjectsClicked
{
	my ($button, $widgets) = @_;
	my @children = $widgets->{'lstBatchList'}->children;
	$widgets->{'lstBatchList'}->remove_items( @children );
}

## start batch encoding
sub btnBatchStartClicked
{
	my ($button, $widgets) = @_;
	my @projects = $widgets->{'lstBatchList'}->children;

	if( $#projects > -1 )
	{
		if( GetBatchStatus() ne "started" )
		{
			$widgets->{'btnBatchStart'}->set_sensitive( $false );
			SetBatchStatus( "started" );
			for( my $i=0; $i<@projects; ++$i )
			{
				if( GetBatchStatus() eq "started" )
				{
					my $item = $projects[$i];
					my @itemchildren = $item->children;
					my $filenamelabel = $itemchildren[0];
					my $filename = $filenamelabel->get;

					$widgets->{'lblBatchActions'}->set_text( "Batch encoding $filename" );
					$widgets->{'lstBatchList'}->select_child( $item );

					ReadProject( $filename, $widgets );
					btnStartClicked($widgets->{'btnStart'}, $widgets);
				}
				else
				{
					$widgets->{'lblBatchActions'}->set_text( "Batch encoding stopped" );
				}
			}
			SetBatchStatus( "stopped" );
			$widgets->{'btnBatchStart'}->set_sensitive( $true );

		}
		else { $widgets->{'lblBatchActions'}->set_text( "Already busy !!" ) }
	}
}

## stop batch encoding
sub btnBatchStopClicked
{
	my ($button,$widgets) = @_;
	SetBatchStatus( "stopped" );
	btnStopClicked( $widgets->{'bntStop'}, $widgets );
}

## save the A/V options to homedir
sub MenuSaveSettings
{
	my ($widgets) = @_;

	## get homedir
	my $home = $ENV{HOME};
	if( !SettingsDirExists() )
	{
		if( !mkdir( $home."/.file2divx3pass" ) )
		{
			MyPrint( $widgets, "Could not create .file2divx3pass in your homedir $home" );
			return $false;
		}
	}

	my $path = $home."/.file2divx3pass/settings";
	my $res = open( FP, ">$path");
	if( !defined($res) )
	{
		MyPrint( $widgets, "Could not write to settingsfile" );
		return $false;
	}

	## audio settings
	print FP "cmbAudioBrate=".$widgets->{'cmbAudioBrate'}->entry->get_text."\n";
	print FP "cmbAudioRate=".$widgets->{'cmbAudioRate'}->entry->get_text."\n";
	## video settings
	print FP "entrySize=".$widgets->{'entrySize'}->get_text."\n";
	print FP "entryScaleWidth=".$widgets->{'entryScaleWidth'}->get_text."\n";
	print FP "entryScaleHeight=".$widgets->{'entryScaleHeight'}->get_text."\n";
	if( $widgets->{'rbtnScale169'}->active )
	{ print FP "aspectratio=16:9\n"; }
	else { print FP "aspectratio=4:3\n"; }
	print FP "cmbCodec=".$widgets->{'cmbCodec'}->entry->get_text."\n";
	print FP "cmbPass=".$widgets->{'cmbPass'}->entry->get_text."\n";
	print FP "entryEncodingOpts=".$widgets->{'entryEncodingOpts'}->get_text."\n";

	close( FP );
	MyPrint( $widgets, "Settings saved" );
}

## load A/V settings from homedir
sub MenuLoadSettings
{
	my ($widgets) = @_;

	## get homedir
	my $home = $ENV{HOME};
	## if settingsdir not exists error
	if( !SettingsDirExists() )
	{
		MyPrint( $widgets, "Could not find your settings", $widgets );
		return $false;
	}

	my $path = $home."/.file2divx3pass/settings";
	my $res = open( FP, "<$path");
	if( !defined($res) )
	{
		MyPrint( $widgets, "Could not read settingsfile", $widgets );
		return $false;
	}
	my $codec = "divx4";
	while( <FP> )
	{
		#my ($k,$v) = split( /=/, $_ ); chop($v);
		my $k = substr( $_, 0, index( $_, "=" ) );
		my $v = substr( $_, index( $_, "=" ) + 1 );
		chop( $v );


		## audio
		if( $k eq "cmbAudioBrate" ) { $widgets->{'cmbAudioBrate'}->entry->set_text( $v ); }
		if( $k eq "cmbAudioRate" ) { $widgets->{'cmbAudioRate'}->entry->set_text( $v ); }

		## video
		if( $k eq "entrySize" ) { $widgets->{'entrySize'}->set_text( $v ); }
		if( $k eq "entryScaleWidth" ) { $widgets->{'entryScaleWidth'}->set_text( $v ); }
		if( $k eq "entryScaleHeight" ) { $widgets->{'entryScaleHeight'}->set_text( $v ); }
		if( $k eq "aspectratio" && $v eq "16:9" ) { $widgets->{'rbtnScale169'}->set_active( $true ); }
		elsif( $k eq "aspectratio" && $v eq "4:3" ) { $widgets->{'rbtnScale43'}->set_active( $true ); }
		if( $k eq "cmbCodec" )
		{
			$widgets->{'cmbCodec'}->entry->set_text( $v );
			$codec = $v;
		}
		if( $k eq "cmbPasspass" ) { $widgets->{'cmbVideoPass'}->entry->set_text( $v ); }
		if( $k eq "entryEncodingOpts" )
		{
			$widgets->{'entryEncodingOpts'}->set_text( $v );
			SetSetting( $codec."extra", $v );			
		}
	}
	close( FP );

	MyPrint( $widgets, "Settings loaded", $widgets );

}

1;


